@@ -4,7 +4,7 @@ import logging
 import os
 from base64 import b64encode
 from copy import copy
-from typing import TYPE_CHECKING, Callable, Optional
+from typing import TYPE_CHECKING, Callable, Optional, Type
 
 from sqlalchemy import MetaData
 from sqlalchemy.ext.asyncio import AsyncEngine, create_async_engine
@@ -60,11 +60,14 @@ class Client(singleton.Singleton):
         cache: If http requests should be cached. It is recommended that you make use of this to not ratelimit yourself as quickly.
         manifest_storage: If manifest entires should be stored, and how. Disabling this stops you from querying the manifest.
         always_return_manifest_information: If manifest information should always be returned on every request. Keep in mind that this will increase requests performed / storage for an increased ease of use.
+        api_client_class: Supply a custom class for `Client.api`
+        http_client_class: Supply a custom class for `Client.http`
+        manifest_client_class: Supply a custom class for `Client.manifest`
     """
 
-    bungie_client_id: str = custom_field(repr=False)
-    bungie_client_secret: str = custom_field(repr=False)
-    bungie_token: str = custom_field(repr=False)
+    bungie_client_id: str = custom_field()
+    bungie_client_secret: str = custom_field()
+    bungie_token: str = custom_field()
 
     logger: logging.Logger = custom_field(default=DEFAULT_LOGGER)
 
@@ -74,14 +77,20 @@ class Client(singleton.Singleton):
     manifest_storage: bool | AsyncEngine = custom_field(default=True)
     always_return_manifest_information: bool = custom_field(default=False)
 
-    json_dumps: Callable = custom_field(init=False, default=json_dumps, repr=False)
-    json_loads: Callable = custom_field(init=False, default=json_loads, repr=False)
+    api_client_class: Type[ApiClient] = custom_field(default=ApiClient)
+    http_client_class: Type[HttpClient] = custom_field(default=HttpClient)
+    manifest_client_class: Type[Manifest] = custom_field(default=Manifest)
 
-    api: ApiClient = custom_field(init=False, repr=False)
-    http: HttpClient = custom_field(init=False, repr=False)
-    manifest: Optional[Manifest] = custom_field(init=False, default=None, repr=False)
+    json_dumps: Callable = custom_field(init=False, default=json_dumps)
+    json_loads: Callable = custom_field(init=False, default=json_loads)
 
-    _metadata: Optional[MetaData] = custom_field(init=False, default=None, repr=False)
+    api: ApiClient = custom_field(init=False)
+    http: HttpClient = custom_field(init=False)
+    manifest: Optional[Manifest] = custom_field(init=False, default=None)
+
+    _metadata: Optional[MetaData] = custom_field(init=False, default=None)
+
+    _tasks: set = custom_field(init=False, factory=set)
 
     _initialised: bool = custom_field(init=False, default=False)
 
@@ -113,10 +122,10 @@ class Client(singleton.Singleton):
         singleton.client = self
 
         # set up the api client
-        self.api = ApiClient()
+        self.api = self.api_client_class()
 
         # set up the http client
-        self.http = HttpClient(
+        self.http = self.http_client_class(
             bungie_headers={
                 "User-Agent": "BungIO",
                 "X-API-Key": self.bungie_token,
@@ -137,7 +146,7 @@ class Client(singleton.Singleton):
                     f"""sqlite+aiosqlite:///{os.path.join(ROOT_DIR, "manifest.db")}"""
                 )
             self._metadata = MetaData(bind=self.manifest_storage)
-            self.manifest = Manifest()
+            self.manifest = self.manifest_client_class()
 
     def get_auth_url(self, state: str) -> str:
         """
@@ -209,13 +218,13 @@ class Client(singleton.Singleton):
                 if not entry.bungie_global_display_name or not entry.bungie_global_display_name_code:
                     auth.bungie_name = None
                 else:
-                    auth.bungie_name = (
-                        f"{entry.bungie_global_display_name}#{str(entry.bungie_global_display_name_code).zfill(4)}"
-                    )
+                    auth.bungie_name = entry.full_bungie_name
         assert auth.membership_type is not MISSING
 
         # dispatch the update event
-        asyncio.create_task(self._client.on_token_update(before=None, after=auth))
+        task = asyncio.create_task(self.on_token_update(before=None, after=auth))
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.discard)
 
         return auth
 
@@ -231,7 +240,9 @@ class Client(singleton.Singleton):
         auth.token = None
 
         # dispatch the update event
-        asyncio.create_task(self.on_token_update(before=old_auth, after=auth))
+        task = asyncio.create_task(self.on_token_update(before=old_auth, after=auth))
+        self._tasks.add(task)
+        task.add_done_callback(self._tasks.discard)
 
     async def get_working_auth(self, auth: AuthData) -> AuthData:
         """
@@ -277,10 +288,12 @@ class Client(singleton.Singleton):
             auth.token = data["access_token"]
             auth.refresh_token = data["refresh_token"]
             auth.token_expiry = now + datetime.timedelta(seconds=data["expires_in"])
-            auth.refresh_expires_in = now + datetime.timedelta(seconds=data["refresh_expires_in"])
+            auth.refresh_token_expiry = now + datetime.timedelta(seconds=data["refresh_expires_in"])
 
             # dispatch the update event
-            asyncio.create_task(self._client.on_token_update(before=old_auth, after=auth))
+            task = asyncio.create_task(self.on_token_update(before=old_auth, after=auth))
+            self._tasks.add(task)
+            task.add_done_callback(self._tasks.discard)
 
         return auth
 
