@@ -1,7 +1,9 @@
 from __future__ import annotations
 
+import asyncio
 import builtins
 import datetime
+import functools
 import inspect
 from enum import Enum, IntFlag
 from functools import partial
@@ -27,6 +29,7 @@ __all__ = (
     "UnknownEnumValue",
     "FuzzyAttrFinder",
     "EnumMixin",
+    "HashObject",
 )
 
 
@@ -35,9 +38,10 @@ custom_define = partial(
     **{
         "slots": True,
         "kw_only": True,
+        "eq": False,
     },
 )
-custom_field = partial(attrs.field, **{})
+custom_field = partial(attrs.field, **{"repr": False})
 
 
 _catch_types = []
@@ -56,15 +60,11 @@ def _enforce_type(field_type: Any, value: Any) -> Any:
     elif field_type in ["Any", Any]:
         pass
     else:
-        # this mean it's not something we want to enforce, so raise an error
+        # this means it's not something we want to enforce, so raise an error
         raise ValueError
     return value
 
 
-# def _transform_builtins(cls, fields: list[attrs.Attribute]) -> list[attrs.Attribute]:
-#     for
-
-
 class MISSING:
     """
     A missing entry which was not returned by bungie whatsoever.
@@ -73,18 +73,47 @@ class MISSING:
     def __bool__(self) -> bool:
         return False
 
+    def __repr__(self) -> str:
+        return "<MISSING>"
+
 
 MISSING = MISSING()
 
 
+@custom_define(slots=False)
+class HashObject:
+    """
+    Many objs have a unique `hash` attribute, this implements logic for this.
+
+    Attributes:
+        hash: The id / hash of the object
+    """
+
+    hash: int = custom_field(repr=True)
+
+    def __eq__(self, other: Any) -> bool:
+        if hasattr(other, "hash"):
+            other = other.hash
+        return self.hash == other
+
+    def __ne__(self, other: Any) -> bool:
+        return self.hash != other.hash
+
+    def __hash__(self) -> int:
+        return self.hash
+
+    def __int__(self) -> int:
+        return self.hash
+
+
 @custom_define()
 class UnknownEnumValue:
     """
     Sometimes Bungie returns information that they have not defined anywhere, so this has to do
     """
 
-    value: int | str = custom_field()
-    enum: Type[BaseEnum] = custom_field()
+    value: int | str = custom_field(repr=True)
+    enum: Type[BaseEnum] = custom_field(repr=True)
 
 
 class EnumMixin(Enum):
@@ -175,7 +204,7 @@ class ClientMixin:
     Mixin that give models access to the client obj
     """
 
-    _client: "Client" = custom_field(repr=False, init=False)
+    _client: "Client" = custom_field(init=False)
 
     @_client.default
     def __client_factory(self) -> "Client":
@@ -289,13 +318,43 @@ class BaseModel(ClientMixin):
 
         return res
 
+    @staticmethod
+    @functools.cache
+    def _acquire_type(field_type: Any, value_is_none: bool) -> Any:
+        if isinstance(field_type, _UnionGenericAlias):
+            field_type = str(field_type)
+
+            # catch optional
+            if "Optional" in field_type:
+                if value_is_none:
+                    raise NameError
+                field_type = (
+                    field_type.removeprefix("typing.Optional[")
+                    .removesuffix("]")
+                    .replace("ForwardRef('", "")
+                    .replace("')", "")
+                )
+
+            # catch union
+            if "Union" in field_type:
+                field_type = (
+                    field_type.removeprefix("typing.Union[")
+                    .removesuffix(", int]")
+                    .replace("ForwardRef('", "")
+                    .replace("')", "")
+                )
+
+        return field_type
+
     @staticmethod
     async def _convert_to_type(field_type: Any, field_metadata: Optional[dict], value: Any, client: Client) -> Any:
-        # catch optional
-        if isinstance(field_type, _UnionGenericAlias) and "Optional" in str(field_type):
-            if value is None:
-                return value
-            field_type = str(field_type).removeprefix("typing.Optional[").removesuffix("]")
+        if value is None:
+            return None
+
+        try:
+            field_type = BaseModel._acquire_type(field_type=field_type, value_is_none=value is None)
+        except NameError:
+            return None
 
         # catch build-ins
         try:
@@ -303,7 +362,10 @@ class BaseModel(ClientMixin):
         except ValueError:
             # sometimes the field type is the attr class as a string
             if isinstance(field_type, str):
-                field_type = getattr(models, field_type)
+                try:
+                    field_type = getattr(models, field_type)
+                except AttributeError:
+                    pass
 
         # convert models in models
         if hasattr(field_type, "from_dict"):
@@ -381,6 +443,7 @@ class BaseModel(ClientMixin):
         return value
 
     @staticmethod
+    @functools.cache
     def _convert_to_bungie_case(string: str) -> str:
         """
         Convert a string to how it is represented by bungie: my_name_string -> myNameString
@@ -392,11 +455,11 @@ class BaseModel(ClientMixin):
             The bungie string
         """
 
-        split = string.split("_")
-        if len(split) == 1:
-            return split[0]
+        if "_" not in string:
+            return string
         else:
-            return "".join([split[0], *[s.capitalize() for s in split[1:]]])
+            split = string.split("_")
+            return "".join((split[0], *(s.capitalize() for s in split[1:])))
 
     def to_dict(self) -> dict:
         """
@@ -440,9 +503,38 @@ class BaseModel(ClientMixin):
 
         return result
 
-    async def fetch_manifest_information(self, _cache: Optional[dict] = None):
+    async def fetch_manifest_information(
+        self, include: Optional[list[str]] = None, exclude: Optional[list[str]] = None, _cache: Optional[dict] = None
+    ):
         """
         Fill the model in-place with information from the manifest.
+
+        Example:
+            Fill every attribute
+            ```py
+            model: DestinyHistoricalStatsActivity
+            await model.fetch_manifest_information()
+            assert model.manifest_director_activity_hash is not None
+            assert model.manifest_reference_id is not None
+            ```
+
+            Fill only some attribute
+            ```py
+            model: DestinyHistoricalStatsActivity
+            await model.fetch_manifest_information(include=["manifest_director_activity_hash"])
+            assert model.manifest_director_activity_hash is not None
+            assert model.manifest_reference_id is None
+            ```
+            ```py
+            model: DestinyHistoricalStatsActivity
+            await model.fetch_manifest_information(exclude=["manifest_director_activity_hash"])
+            assert model.manifest_director_activity_hash is None
+            assert model.manifest_reference_id is not None
+            ```
+
+        Args:
+            include: A list of attributes you want to include. Excludes everything not mentioned
+            exclude:  A list of attributes you want to exclude. Includes everything not mentioned
         """
 
         if not isinstance(self._client.manifest_storage, AsyncEngine):
@@ -451,13 +543,18 @@ class BaseModel(ClientMixin):
         if not _cache:
             _cache = {}
 
-        class_definition = fields_dict(type(self))  # noqa
+        class_definition = attrs.fields_dict(type(self))  # noqa
 
         # loop through the class attributes
         for name in self.__dir__():
             if name.startswith("__"):
                 continue
 
+            if include and name not in include:
+                continue
+            if exclude and name in exclude:
+                continue
+
             # manifest entries
             if name.startswith("manifest_"):
                 striped_name = name.removeprefix("manifest_")
